@@ -61,38 +61,48 @@ as possible second argument) to the desired representation of date/time/timestam
     (incf offset))
   offset)
 
+(defmacro with-allocate-foreign-string ((var len) &body body)
+  "Safely does uffi:allocate-foreign-string-- making sure we do the uffi:free-foreign-object"
+  `(let ((,var))
+    (unwind-protect
+         (progn
+           (setf ,var (uffi:allocate-foreign-string ,len))
+           ,@body)
+      (when ,var
+        (uffi:free-foreign-object ,var)))))
+
+(defmacro with-allocate-foreign-strings (bindings &rest body)
+  (if bindings
+      `(with-allocate-foreign-string ,(car bindings)
+        (with-allocate-foreign-strings ,(cdr bindings)
+          ,@body))
+      `(progn ,@body)))
+
 (defun handle-error (henv hdbc hstmt)
-  (let ((sql-state (allocate-foreign-string 256))
-        (error-message (allocate-foreign-string #.$SQL_MAX_MESSAGE_LENGTH)))
-    (with-foreign-objects ((error-code #.$ODBC-LONG-TYPE)
-                           (msg-length :short))
-      (SQLError henv hdbc hstmt sql-state
-                error-code error-message
-                #.$SQL_MAX_MESSAGE_LENGTH msg-length)
-      (let ((err (convert-from-foreign-string error-message))
-            (state (convert-from-foreign-string sql-state)))
-        (free-foreign-object error-message)
-        (free-foreign-object sql-state)
-        (values
-         err
-         state
-         (deref-pointer msg-length :short)
-         (deref-pointer error-code #.$ODBC-LONG-TYPE))))))
+  (with-allocate-foreign-strings ((sql-state 256)
+                                  (error-message #.$SQL_MAX_MESSAGE_LENGTH))
+   (with-foreign-objects ((error-code #.$ODBC-LONG-TYPE)
+                          (msg-length :short))
+    (SQLError henv hdbc hstmt sql-state
+              error-code error-message
+              #.$SQL_MAX_MESSAGE_LENGTH msg-length)
+    (values
+     (convert-from-foreign-string error-message)
+     (convert-from-foreign-string sql-state)
+     (deref-pointer msg-length :short)
+     (deref-pointer error-code #.$ODBC-LONG-TYPE)))))
 
 (defun sql-state (henv hdbc hstmt)
-  (let ((sql-state (allocate-foreign-string 256))
-        (error-message (allocate-foreign-string #.$SQL_MAX_MESSAGE_LENGTH)))
-    (with-foreign-objects ((error-code #.$ODBC-LONG-TYPE)
-                           (msg-length :short))
-      (SQLError henv hdbc hstmt sql-state error-code
-                error-message #.$SQL_MAX_MESSAGE_LENGTH msg-length)
-      (let ((state (convert-from-foreign-string sql-state)))
-        (free-foreign-object error-message)
-        (free-foreign-object sql-state)
-        state
-        ;; test this: return a keyword for efficiency
-        ;;(%cstring-to-keyword state)
-        ))))
+  (with-allocate-foreign-strings ((sql-state 256)
+                                  (error-message #.$SQL_MAX_MESSAGE_LENGTH))
+   (with-foreign-objects ((error-code #.$ODBC-LONG-TYPE)
+                          (msg-length :short))
+    (SQLError henv hdbc hstmt sql-state error-code
+              error-message #.$SQL_MAX_MESSAGE_LENGTH msg-length)
+    (convert-from-foreign-string sql-state)
+    ;; test this: return a keyword for efficiency
+    ;;(%cstring-to-keyword state)
+    )))
 
 (defmacro with-error-handling ((&key henv hdbc hstmt (print-info t))
                                    odbc-call &body body)
@@ -209,23 +219,24 @@ as possible second argument) to the desired representation of date/time/timestam
 
 (defun %sql-driver-connect (hdbc connection-string completion window-handle)
   (with-cstring (connection-ptr connection-string)
-    (let ((completed-connection-string (allocate-foreign-string $SQL_MAX_CONN_OUT)))
-      (unwind-protect
-          (with-foreign-object (completed-connection-length :short)
-                               (with-error-handling
-                                   (:hdbc hdbc)
-                                   (SQLDriverConnect hdbc
-                                                     window-handle
-                                                     connection-ptr $SQL_NTS
-                                                     completed-connection-string $SQL_MAX_CONN_OUT
-                                                     completed-connection-length
-                                                     completion)))
-        (free-foreign-object completed-connection-string)))))
+    (with-allocate-foreign-string (completed-connection-string-ptr $SQL_MAX_CONN_OUT)
+      (with-foreign-object (completed-connection-length :short)
+       (with-error-handling
+           (:hdbc hdbc)
+           (SQLDriverConnect hdbc
+                             window-handle
+                             connection-ptr $SQL_NTS
+                             completed-connection-string-ptr $SQL_MAX_CONN_OUT
+                             completed-connection-length
+                             completion))))))
 
 (defun %disconnect (hdbc)
   (with-error-handling
     (:hdbc hdbc)
-    (SQLDisconnect hdbc)))
+    (SQLDisconnect hdbc)
+    (with-error-handling
+        (:hdbc hdbc)
+        (SQLFreeHandle $SQL_HANDLE_DBC hdbc))))
 
 (defun %commit (henv hdbc)
   (with-error-handling
@@ -239,7 +250,7 @@ as possible second argument) to the desired representation of date/time/timestam
     (SQLTransact
      henv hdbc $SQL_ROLLBACK)))
 
-; col-nr is zero-based in Lisp
+; col-nr is zero-based in Lisp but 1 based in sql
 ; col-nr = :bookmark retrieves a bookmark.
 (defun %bind-column (hstmt column-nr c-type data-ptr precision out-len-ptr)
   (with-error-handling
@@ -317,14 +328,12 @@ as possible second argument) to the desired representation of date/time/timestam
       #.$SQL_SPECIAL_CHARACTERS
       #.$SQL_TABLE_TERM
       #.$SQL_USER_NAME)
-     (let ((info-ptr (allocate-foreign-string 1024)))
+     (with-allocate-foreign-string (info-ptr 1024)
        (with-foreign-object (info-length-ptr :short)
-         (with-error-handling
-             (:hdbc hdbc)
-             (SQLGetInfo hdbc info-type info-ptr 1023 info-length-ptr)
-           (let ((info (convert-from-foreign-string info-ptr)))
-             (free-foreign-object info-ptr)
-             info)))))
+        (with-error-handling
+            (:hdbc hdbc)
+            (SQLGetInfo hdbc info-type info-ptr 1023 info-length-ptr)
+          (convert-from-foreign-string info-ptr)))))
     ;; those returning a word
     ((#.$SQL_ACTIVE_CONNECTIONS
       #.$SQL_ACTIVE_STATEMENTS
@@ -470,29 +479,28 @@ as possible second argument) to the desired representation of date/time/timestam
 
 ;; column counting is 1-based
 (defun %describe-column (hstmt column-nr)
-  (let ((column-name-ptr (allocate-foreign-string 256)))
+  (with-allocate-foreign-string (column-name-ptr 256)
     (with-foreign-objects ((column-name-length-ptr :short)
                            (column-sql-type-ptr :short)
                            (column-precision-ptr #.$ODBC-ULONG-TYPE)
                            (column-scale-ptr :short)
                            (column-nullable-p-ptr :short))
-      (with-error-handling (:hstmt hstmt)
-          (SQLDescribeCol hstmt column-nr column-name-ptr 256
-                          column-name-length-ptr
-                          column-sql-type-ptr
-                          column-precision-ptr
-                          column-scale-ptr
-                          column-nullable-p-ptr)
-        (let ((column-name (convert-from-foreign-string column-name-ptr)))
-          (free-foreign-object column-name-ptr)
-          (values
-           column-name
-           (deref-pointer column-sql-type-ptr :short)
-           (deref-pointer column-precision-ptr #.$ODBC-ULONG-TYPE)
-           (deref-pointer column-scale-ptr :short)
-           (deref-pointer column-nullable-p-ptr :short)))))))
+     (with-error-handling (:hstmt hstmt)
+         (SQLDescribeCol hstmt column-nr column-name-ptr 256
+                         column-name-length-ptr
+                         column-sql-type-ptr
+                         column-precision-ptr
+                         column-scale-ptr
+                         column-nullable-p-ptr)
+       (values
+        (convert-from-foreign-string column-name-ptr)
+        (deref-pointer column-sql-type-ptr :short)
+        (deref-pointer column-precision-ptr #.$ODBC-ULONG-TYPE)
+        (deref-pointer column-scale-ptr :short)
+        (deref-pointer column-nullable-p-ptr :short))))))
 
 ;; parameter counting is 1-based
+;; this function isn't used, which is good because FreeTDS dosn't support it.
 (defun %describe-parameter (hstmt parameter-nr)
   (with-foreign-objects ((column-sql-type-ptr :short)
                          (column-precision-ptr #.$ODBC-ULONG-TYPE)
@@ -512,19 +520,17 @@ as possible second argument) to the desired representation of date/time/timestam
        (deref-pointer column-nullable-p-ptr :short)))))
 
 (defun %column-attributes (hstmt column-nr descriptor-type)
-  (let ((descriptor-info-ptr (allocate-foreign-string 256)))
+  (with-allocate-foreign-string (descriptor-info-ptr 256)
     (with-foreign-objects ((descriptor-length-ptr :short)
                            (numeric-descriptor-ptr #.$ODBC-LONG-TYPE))
-      (with-error-handling
-          (:hstmt hstmt)
-          (SQLColAttributes hstmt column-nr descriptor-type descriptor-info-ptr
-                            256 descriptor-length-ptr
-                            numeric-descriptor-ptr)
-        (let ((desc (convert-from-foreign-string descriptor-info-ptr)))
-          (free-foreign-object descriptor-info-ptr)
-          (values
-           desc
-           (deref-pointer numeric-descriptor-ptr #.$ODBC-LONG-TYPE)))))))
+     (with-error-handling
+         (:hstmt hstmt)
+         (SQLColAttributes hstmt column-nr descriptor-type descriptor-info-ptr
+                           256 descriptor-length-ptr
+                           numeric-descriptor-ptr)
+       (values
+        (convert-from-foreign-string descriptor-info-ptr)
+        (deref-pointer numeric-descriptor-ptr #.$ODBC-LONG-TYPE))))))
 
 (defun %prepare-describe-columns (hstmt table-qualifier table-owner
                                    table-name column-name)
@@ -548,43 +554,35 @@ as possible second argument) to the desired representation of date/time/timestam
     (fetch-all-rows hstmt)))
 
 (defun %sql-data-sources (henv &key (direction :first))
-  (let ((name-ptr (allocate-foreign-string (1+ $SQL_MAX_DSN_LENGTH)))
-        (description-ptr (allocate-foreign-string 1024)))
-    (with-foreign-objects ((name-length-ptr :short)
-                           (description-length-ptr :short))
-      (let ((res (with-error-handling
-                     (:henv henv)
-                     (SQLDataSources henv
-                                     (ecase direction
-                                       (:first $SQL_FETCH_FIRST)
-                                       (:next $SQL_FETCH_NEXT))
-                                     name-ptr
-                                     (1+ $SQL_MAX_DSN_LENGTH)
-                                     name-length-ptr
-                                     description-ptr
-                                     1024
-                                     description-length-ptr))))
-        (cond
-          ((= res $SQL_NO_DATA_FOUND)
-           (let ((name (convert-from-foreign-string name-ptr))
-                 (desc (convert-from-foreign-string description-ptr)))
-             (free-foreign-object name-ptr)
-             (free-foreign-object description-ptr)
-             (values
-              name
-              desc)))
-          (t
-           (free-foreign-object name-ptr)
-           (free-foreign-object description-ptr)
-           nil))))))
+  (with-allocate-foreign-strings ((name-ptr (1+ $SQL_MAX_DSN_LENGTH))
+                                  (description-ptr 1024))
+   (with-foreign-objects ((name-length-ptr :short)
+                          (description-length-ptr :short))
+    (let ((res (with-error-handling
+                   (:henv henv)
+                   (SQLDataSources henv
+                                   (ecase direction
+                                     (:first $SQL_FETCH_FIRST)
+                                     (:next $SQL_FETCH_NEXT))
+                                   name-ptr
+                                   (1+ $SQL_MAX_DSN_LENGTH)
+                                   name-length-ptr
+                                   description-ptr
+                                   1024
+                                   description-length-ptr))))
+      (when (= res $SQL_NO_DATA_FOUND)
+        (values
+          (convert-from-foreign-string name-ptr)
+          (convert-from-foreign-string description-ptr)))))))
 
 
 
 (defun sql-to-c-type (sql-type)
   (ecase sql-type
     ((#.$SQL_CHAR #.$SQL_VARCHAR #.$SQL_LONGVARCHAR
-      #.$SQL_NUMERIC #.$SQL_DECIMAL #.$SQL_BIGINT -8 -9 -10) $SQL_C_CHAR) ;; Added -10 for MSSQL ntext type
+      #.$SQL_NUMERIC #.$SQL_DECIMAL -8 -9 -10) $SQL_C_CHAR) ;; Added -10 for MSSQL ntext type
     (#.$SQL_INTEGER $SQL_C_SLONG)
+    (#.$SQL_BIGINT $SQL_C_SBIGINT)
     (#.$SQL_SMALLINT $SQL_C_SSHORT)
     (#.$SQL_DOUBLE $SQL_C_DOUBLE)
     (#.$SQL_FLOAT $SQL_C_DOUBLE)
@@ -603,6 +601,7 @@ as possible second argument) to the desired representation of date/time/timestam
 (def-type short-pointer-type (* :short))
 (def-type int-pointer-type (* :int))
 (def-type long-pointer-type (* #.$ODBC-LONG-TYPE))
+(def-type big-pointer-type (* #.$ODBC-BIG-TYPE))
 (def-type float-pointer-type (* :float))
 (def-type double-pointer-type (* :double))
 (def-type string-pointer-type (* :unsigned-char))
@@ -623,6 +622,10 @@ as possible second argument) to the desired representation of date/time/timestam
   (locally (declare (type long-pointer-type ptr))
     (deref-pointer ptr #.$ODBC-LONG-TYPE)))
 
+(defun get-cast-big (ptr)
+  (locally (declare (type big-pointer-type ptr))
+    (deref-pointer ptr #.$ODBC-BIG-TYPE)))
+
 (defun get-cast-single-float (ptr)
   (locally (declare (type float-pointer-type ptr))
     (deref-pointer ptr :float)))
@@ -669,8 +672,7 @@ as possible second argument) to the desired representation of date/time/timestam
                    (#.$SQL_C_SSHORT (get-cast-short data-ptr)) ;; ?
                    (#.$SQL_SMALLINT (get-cast-short data-ptr)) ;; ??
                    (#.$SQL_INTEGER (get-cast-int data-ptr))
-                   (#.$SQL_BIGINT (read-from-string
-                                   (get-cast-foreign-string data-ptr)))
+                   (#.$SQL_BIGINT (get-cast-big data-ptr))
                    (#.$SQL_DECIMAL
                     (let ((*read-base* 10))
                       (read-from-string (get-cast-foreign-string data-ptr))))
@@ -700,6 +702,7 @@ as possible second argument) to the desired representation of date/time/timestam
                        (get-cast-binary data-ptr out-len *binary-format*))
                       ((#.$SQL_C_SSHORT #.$SQL_C_STINYINT) ; LMH short ints
                        (get-cast-short data-ptr)) ; LMH
+                      (#.$SQL_C_SBIGINT (get-cast-big data-ptr))
                       #+ignore
                       (#.$SQL_C_CHAR
                        (code-char (get-cast-short data-ptr)))
@@ -739,6 +742,7 @@ as possible second argument) to the desired representation of date/time/timestam
             (#.$SQL_C_DOUBLE (uffi:allocate-foreign-object :double))
             (#.$SQL_C_BIT (uffi:allocate-foreign-object :byte))
             (#.$SQL_C_STINYINT (uffi:allocate-foreign-object :byte))
+            (#.$SQL_C_SBIGINT (uffi:allocate-foreign-object #.$ODBC-BIG-TYPE))
             (#.$SQL_C_SSHORT (uffi:allocate-foreign-object :short))
             (#.$SQL_C_CHAR (uffi:allocate-foreign-string (1+ size)))
             (#.$SQL_C_BINARY (uffi:allocate-foreign-string (1+ (* 2 size))))
@@ -913,9 +917,7 @@ as possible second argument) to the desired representation of date/time/timestam
                                 finally (incf offset (1- i)))
                          while
                            (and (= res $SQL_SUCCESS_WITH_INFO)
-                                #+ingore(eq (sql-state +null-handle-ptr+ +null-handle-ptr+ hstmt)
-                                            $sql-data-truncated)
-                                (equal (sql-state +null-handle-ptr+ +null-handle-ptr+ hstmt) "01004"))
+                                (> out-len +max-precision+))
                          do
                            (setf res (%sql-get-data hstmt column-nr c-type data-ptr +max-precision+ out-len-ptr)
                                  out-len (deref-pointer out-len-ptr #.$ODBC-LONG-TYPE)))
@@ -944,6 +946,7 @@ as possible second argument) to the desired representation of date/time/timestam
    (get-slot-value ptr 'sql-c-timestamp 'fraction)))
 
 (defun universal-time-to-timestamp (time &optional (fraction 0))
+  "TODO: Dead function?"
   (multiple-value-bind (sec min hour day month year)
       (decode-universal-time time)
     (let ((ptr (allocate-foreign-object 'sql-c-timestamp)))
@@ -957,6 +960,7 @@ as possible second argument) to the desired representation of date/time/timestam
       ptr)))
 
 (defun %put-timestamp (ptr time &optional (fraction 0))
+  "TODO: Dead function?"
   (declare (type c-timestamp-ptr-type ptr))
   (multiple-value-bind (sec min hour day month year)
       (decode-universal-time time)
@@ -990,6 +994,9 @@ as possible second argument) to the desired representation of date/time/timestam
 
 (defun %set-attr-odbc-version (henv version)
   (with-error-handling (:henv henv)
+      ;;note that we are passing version as an integer that happens to be
+      ;;stuffed into a pointer.
+      ;;http://msdn.microsoft.com/en-us/library/ms709285%28v=VS.85%29.aspx
       (SQLSetEnvAttr henv $SQL_ATTR_ODBC_VERSION
                      (make-pointer version :void) 0)))
 
@@ -1009,34 +1016,28 @@ as possible second argument) to the desired representation of date/time/timestam
        (if ensure $SQL_ENSURE $SQL_QUICK)))))
 
 (defun %list-data-sources (henv)
-  (let ((dsn (allocate-foreign-string (1+ $SQL_MAX_DSN_LENGTH)))
-        (desc (allocate-foreign-string 256))
-        (results nil))
-    (unwind-protect
-         (with-foreign-objects ((dsn-len :short)
-                                (desc-len :short))
-           (let ((res (with-error-handling (:henv henv)
-                        (SQLDataSources henv $SQL_FETCH_FIRST dsn
-                                        (1+ $SQL_MAX_DSN_LENGTH)
-                                        dsn-len desc 256 desc-len))))
-             (when (or (eql res $SQL_SUCCESS)
-                       (eql res $SQL_SUCCESS_WITH_INFO))
-               (push (convert-from-foreign-string dsn) results))
+  (let ((results nil))
+    (with-foreign-strings ((dsn-ptr (1+ $SQL_MAX_DSN_LENGTH))
+                           (desc-ptr 256))
+      (with-foreign-objects ((dsn-len :short)
+                             (desc-len :short))
+       (let ((res (with-error-handling (:henv henv)
+                      (SQLDataSources henv $SQL_FETCH_FIRST dsn-ptr
+                                      (1+ $SQL_MAX_DSN_LENGTH)
+                                      dsn-len desc-ptr 256 desc-len))))
+         (when (or (eql res $SQL_SUCCESS)
+                   (eql res $SQL_SUCCESS_WITH_INFO))
+           (push (convert-from-foreign-string dsn-ptr) results))
 
-             (do ((res (with-error-handling (:henv henv)
-                         (SQLDataSources henv $SQL_FETCH_NEXT dsn
-                                         (1+ $SQL_MAX_DSN_LENGTH)
-                                         dsn-len desc 256 desc-len))
-                       (with-error-handling (:henv henv)
-                         (SQLDataSources henv $SQL_FETCH_NEXT dsn
-                                         (1+ $SQL_MAX_DSN_LENGTH)
-                                         dsn-len desc 256 desc-len))))
-                 ((not (or (eql res $SQL_SUCCESS)
-                           (eql res $SQL_SUCCESS_WITH_INFO))))
-               (push (convert-from-foreign-string dsn) results))))
-      (progn
-        (free-foreign-object dsn)
-        (free-foreign-object desc)))
+         (do ((res (with-error-handling (:henv henv)
+                       (SQLDataSources henv $SQL_FETCH_NEXT dsn-ptr
+                                       (1+ $SQL_MAX_DSN_LENGTH)
+                                       dsn-len desc-ptr 256 desc-len))
+                   (with-error-handling (:henv henv)
+                       (SQLDataSources henv $SQL_FETCH_NEXT dsn-ptr
+                                       (1+ $SQL_MAX_DSN_LENGTH)
+                                       dsn-len desc-ptr 256 desc-len))))
+             ((not (or (eql res $SQL_SUCCESS)
+                       (eql res $SQL_SUCCESS_WITH_INFO))))
+           (push (convert-from-foreign-string dsn-ptr) results)))))
     (nreverse results)))
-
-
